@@ -294,6 +294,7 @@ int xsane_back_gtk_make_path(size_t buf_size, char *buf, const char *prog_name,
 
     buf[len++] = SLASH; /* OS/2 does not like slash at end of mktemp-path */
   }
+
   if (len >= buf_size)
   {
     goto filename_too_long;
@@ -431,15 +432,7 @@ int xsane_back_gtk_make_path(size_t buf_size, char *buf, const char *prog_name,
     len += 6;
     buf[len] = '\0';
 
-#if 1
     fd = mkstemp(buf); /* create unique filename and opens/creates the file */
-#else
-    mktemp(buf); /* not safe */
-    umask(0177);
-    fd = open(buf, O_WRONLY | O_EXCL | O_CREAT, 0600);
-    umask(XSANE_DEFAULT_UMASK); /* define new file permissions */
-#endif
-
     if (fd == -1)
     {
       xsane_back_gtk_error(ERR_CREATE_TEMP_FILE, FALSE);
@@ -470,7 +463,7 @@ void xsane_back_gtk_set_option(int opt_num, void *val, SANE_Action action)
  SANE_Status status;
  SANE_Int info;
  char buf[TEXTBUFSIZE];
- int old_colors = xsane.xsane_colors;
+ int old_channels = xsane.xsane_channels;
  int update_gamma = FALSE;
 
   DBG(DBG_proc, "xsane_back_gtk_set_option\n");
@@ -505,7 +498,7 @@ void xsane_back_gtk_set_option(int opt_num, void *val, SANE_Action action)
     /* XXXXXXXXXXXXXX this also has to be handled XXXXXXXXXXXXXXX */
   }
 
-  if (xsane.xsane_colors != old_colors)
+  if (xsane.xsane_channels != old_channels)
   {
     /* we have to update gamma tables and histogram because medium settings */
     /* may have changed */
@@ -716,6 +709,7 @@ gint xsane_back_gtk_decision(gchar *title, gchar **xpm_d,  gchar *message, gchar
     gtk_box_pack_start(GTK_BOX(hbox), pixmapwidget, FALSE, FALSE, 10);
     gtk_widget_show(pixmapwidget);
     gdk_drawable_unref(pixmap);
+    gdk_drawable_unref(mask);
   }
 
   /* the message */
@@ -734,7 +728,7 @@ gint xsane_back_gtk_decision(gchar *title, gchar **xpm_d,  gchar *message, gchar
   button = gtk_button_new_with_label(oktext);
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_back_gtk_decision_ok_callback, (void *) decision_flag_ptr);
-  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 5);
+  gtk_box_pack_end(GTK_BOX(hbox), button, TRUE, TRUE, 5);
   gtk_widget_grab_default(button);
   gtk_widget_show(button);
 
@@ -743,7 +737,7 @@ gint xsane_back_gtk_decision(gchar *title, gchar **xpm_d,  gchar *message, gchar
   {
     button = gtk_button_new_with_label(rejecttext);
     g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_back_gtk_decision_reject_callback, (void *) decision_flag_ptr);
-    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 5);
+    gtk_box_pack_end(GTK_BOX(hbox), button, TRUE, TRUE, 5);
     gtk_widget_show(button);
   }
 
@@ -871,59 +865,6 @@ void xsane_back_gtk_info(gchar *info, int wait)
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
-GtkWidget *fileselection;
-char *fileselection_filetype = NULL;
-
-static void xsane_back_gtk_filetype_callback(GtkWidget *widget, gpointer data)
-{
- char *extension, *filename;
- char buffer[PATH_MAX];
- char *new_filetype = (char *) data;
-
-  DBG(DBG_proc, "xsane_filetype_callback\n");
-
-  filename = strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselection)));
-
-  if ((new_filetype) && (*new_filetype))
-  {
-    extension = strrchr(filename, '.');
-
-    if ((extension) && (extension != filename))
-    {
-      if ( (!strcasecmp(extension, ".pnm"))  || (!strcasecmp(extension, ".raw"))
-        || (!strcasecmp(extension, ".png"))  || (!strcasecmp(extension, ".ps"))
-        || (!strcasecmp(extension, ".pdf"))  || (!strcasecmp(extension, ".rgba"))
-        || (!strcasecmp(extension, ".tiff")) || (!strcasecmp(extension, ".tif"))
-        || (!strcasecmp(extension, ".text")) || (!strcasecmp(extension, ".txt"))
-        || (!strcasecmp(extension, ".jpg"))  || (!strcasecmp(extension, ".jpeg"))
-         ) /* remove filetype extension */
-      {
-        *extension = 0; /* remove extension */
-      }
-    }
-    snprintf(buffer, sizeof(buffer), "%s%s", filename, new_filetype);
-    free(filename);
-    filename = strdup(buffer);
-  }
-
-  if (fileselection_filetype)
-  {
-    free(fileselection_filetype);
-    fileselection_filetype = NULL;
-  }
-
-  if (data)
-  {
-    fileselection_filetype = strdup(new_filetype); 
-  }
-
-  gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), filename);
-
-  free(filename);
-}
-
-/* ---------------------------------------------------------------------------------------------------------------------- */
-
 void xsane_back_gtk_filetype_menu_set_history(GtkWidget *xsane_filetype_option_menu, char *filetype)
 {
  int filetype_nr;
@@ -990,7 +931,6 @@ void xsane_back_gtk_filetype_menu_set_history(GtkWidget *xsane_filetype_option_m
     select_item = filetype_nr;
   }
 #endif
-
   gtk_option_menu_set_history(GTK_OPTION_MENU(xsane_filetype_option_menu), select_item);
 }
 
@@ -1113,6 +1053,475 @@ GtkWidget *xsane_back_gtk_filetype_menu_new(char *filetype, GtkSignalFunc filety
 
 /* ----------------------------------------------------------------------------------------------------------------- */
 
+GtkWidget *xsane_back_gtk_cms_function_menu_new(int select_cms_function, GtkSignalFunc cms_function_menu_callback)
+{
+ GtkWidget *xsane_cms_function_menu, *xsane_cms_function_item;
+ GtkWidget *xsane_cms_function_option_menu;
+
+  xsane_cms_function_menu = gtk_menu_new();
+
+  xsane_cms_function_item = gtk_menu_item_new_with_label(MENU_ITEM_CMS_FUNCTION_EMBED_SCANNER_ICM_PROFILE);
+  if (cms_function_menu_callback)
+  {
+    g_signal_connect(GTK_OBJECT(xsane_cms_function_item), "activate", (GtkSignalFunc) cms_function_menu_callback, (void *) XSANE_CMS_FUNCTION_EMBED_SCANNER_ICM_PROFILE);
+  }
+  gtk_container_add(GTK_CONTAINER(xsane_cms_function_menu), xsane_cms_function_item);
+  gtk_widget_show(xsane_cms_function_item);
+
+  xsane_cms_function_item = gtk_menu_item_new_with_label(MENU_ITEM_CMS_FUNCTION_CONVERT_TO_SRGB);
+  if (cms_function_menu_callback)
+  {
+    g_signal_connect(GTK_OBJECT(xsane_cms_function_item), "activate", (GtkSignalFunc) cms_function_menu_callback, (void *) XSANE_CMS_FUNCTION_CONVERT_TO_SRGB);
+  }
+  gtk_container_add(GTK_CONTAINER(xsane_cms_function_menu), xsane_cms_function_item);
+  gtk_widget_show(xsane_cms_function_item);
+
+  xsane_cms_function_item = gtk_menu_item_new_with_label(MENU_ITEM_FUNCTION_CONVERT_TO_WORKING_CS);
+  if (cms_function_menu_callback)
+  {
+    g_signal_connect(GTK_OBJECT(xsane_cms_function_item), "activate", (GtkSignalFunc) cms_function_menu_callback, (void *) XSANE_CMS_FUNCTION_CONVERT_TO_WORKING_CS);
+  }
+  gtk_container_add(GTK_CONTAINER(xsane_cms_function_menu), xsane_cms_function_item);
+  gtk_widget_show(xsane_cms_function_item);
+
+  xsane_cms_function_option_menu = gtk_option_menu_new();
+  xsane_back_gtk_set_tooltip(xsane.tooltips, xsane_cms_function_option_menu, DESC_CMS_FUNCTION);
+  gtk_option_menu_set_menu(GTK_OPTION_MENU(xsane_cms_function_option_menu), xsane_cms_function_menu);
+  gtk_option_menu_set_history(GTK_OPTION_MENU(xsane_cms_function_option_menu), select_cms_function);
+
+ return (xsane_cms_function_option_menu);
+}
+
+/* ----------------------------------------------------------------------------------------------------------------- */
+
+#ifdef __GTK_FILE_CHOOSER_H__
+
+GtkWidget *filechooser;
+char *filechooser_filetype = NULL;
+
+static void xsane_back_gtk_filetype2_callback(GtkWidget *widget, gpointer data)
+{
+ char *extension, *chooser_filename;
+ char filename[PATH_MAX];
+ char *basename;
+ char *new_filetype = (char *) data;
+ int pos;
+
+  DBG(DBG_proc, "xsane_filetype2_callback\n");
+
+  chooser_filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filechooser));
+
+  if ((new_filetype) && (*new_filetype))
+  {
+    extension = strrchr(chooser_filename, '.');
+
+    if ((extension) && (extension != chooser_filename))
+    {
+      if ( (!strcasecmp(extension, ".pnm"))  || (!strcasecmp(extension, ".raw"))
+        || (!strcasecmp(extension, ".png"))  || (!strcasecmp(extension, ".ps"))
+        || (!strcasecmp(extension, ".pdf"))  || (!strcasecmp(extension, ".rgba"))
+        || (!strcasecmp(extension, ".tiff")) || (!strcasecmp(extension, ".tif"))
+        || (!strcasecmp(extension, ".text")) || (!strcasecmp(extension, ".txt"))
+        || (!strcasecmp(extension, ".jpg"))  || (!strcasecmp(extension, ".jpeg"))
+         ) /* remove filetype extension */
+      {
+        *extension = 0; /* remove extension */
+      }
+    }
+    snprintf(filename, sizeof(filename), "%s%s", chooser_filename, new_filetype);
+  }
+
+  if (filechooser_filetype)
+  {
+    free(filechooser_filetype);
+    filechooser_filetype = NULL;
+  }
+
+  if (data)
+  {
+    filechooser_filetype = strdup(new_filetype); 
+  }
+
+
+  basename = filename;
+
+  for (pos = strlen(filename) - 1; pos > 0; pos--)
+  {
+    if (filename[pos] == '/')
+    {
+      filename[pos]=0;
+
+      basename = filename+pos+1;
+     break;
+    }
+  }
+
+  gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filechooser), basename);
+
+  g_free(chooser_filename);
+}
+
+/* ----------------------------------------------------------------------------------------------------------------- */
+
+int xsane_back_gtk_get_filename(const char *label, const char *default_name, size_t max_len, char *filename, char **filetype, int *cms_function,
+                                XsaneFileChooserAction action, int show_extra_widgets, int enable_filters, int activate_filter)
+{
+ int ok = 0;
+ GtkWidget *xsane_filetype_option_menu;
+ GtkWidget *xsane_cms_function_option_menu = xsane_cms_function_option_menu;
+ gint result;
+ const gchar *accept_text = NULL;
+ const gchar *reject_text = NULL;
+ GtkFileChooserAction chooser_action = GTK_FILE_CHOOSER_ACTION_OPEN;
+ GtkResponseType accept_code;
+ GtkResponseType reject_code;
+ char buf[PATH_MAX];
+
+
+  DBG(DBG_proc, "xsane_back_gtk_get_filename\n");
+
+  if (filechooser)
+  {
+    gdk_beep();
+    return -1; /* cancel => do not allow to open more than one filechooser dialog */
+  }
+
+  switch (action)
+  {
+    default:
+    case XSANE_FILE_CHOOSER_ACTION_OPEN:
+      chooser_action = GTK_FILE_CHOOSER_ACTION_OPEN;
+      accept_text = GTK_STOCK_OPEN;
+      accept_code = GTK_RESPONSE_ACCEPT;
+      reject_text = GTK_STOCK_CANCEL;
+      reject_code = GTK_RESPONSE_CANCEL;
+     break;
+
+    case XSANE_FILE_CHOOSER_ACTION_SELECT_OPEN:
+      chooser_action = GTK_FILE_CHOOSER_ACTION_OPEN;
+      accept_text = GTK_STOCK_OK;
+      accept_code = GTK_RESPONSE_ACCEPT;
+      reject_text = GTK_STOCK_CANCEL;
+      reject_code = GTK_RESPONSE_CANCEL;
+     break;
+
+    case XSANE_FILE_CHOOSER_ACTION_SAVE:
+      chooser_action = GTK_FILE_CHOOSER_ACTION_SAVE;
+      accept_text = GTK_STOCK_SAVE;
+      accept_code = GTK_RESPONSE_ACCEPT;
+      reject_text = GTK_STOCK_CANCEL;
+      reject_code = GTK_RESPONSE_CANCEL;
+     break;
+
+    case XSANE_FILE_CHOOSER_ACTION_SELECT_SAVE:
+      chooser_action = GTK_FILE_CHOOSER_ACTION_SAVE;
+      accept_text = GTK_STOCK_OK;
+      accept_code = GTK_RESPONSE_ACCEPT;
+      reject_text = GTK_STOCK_CANCEL;
+      reject_code = GTK_RESPONSE_CANCEL;
+     break;
+
+    case XSANE_FILE_CHOOSER_ACTION_SELECT_FOLDER:
+      chooser_action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
+      accept_text = GTK_STOCK_OK;
+      accept_code = GTK_RESPONSE_ACCEPT;
+      reject_text = GTK_STOCK_CANCEL;
+      reject_code = GTK_RESPONSE_CANCEL;
+     break;
+
+    case XSANE_FILE_CHOOSER_ACTION_SELECT_PROJECT:
+      chooser_action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
+      accept_text = GTK_STOCK_OK;
+      accept_code = GTK_RESPONSE_NO; /* when we would use ACCEPT, OK, YES or APPLY then the filechooser_dialog would create non existant directories */
+      reject_text = GTK_STOCK_CANCEL;
+      reject_code = GTK_RESPONSE_CANCEL;
+     break;
+  }
+
+  filechooser = gtk_file_chooser_dialog_new (label,
+				      NULL,
+				      chooser_action,
+				      reject_text, reject_code,
+				      accept_text, accept_code,
+				      NULL);
+
+  xsane_set_window_icon(filechooser, 0);
+
+  gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(filechooser), TRUE);
+
+
+  /* add paths to filechooser */
+  if (getenv(STRINGIFY(ENVIRONMENT_HOME_DIR_NAME)) != NULL)
+  {
+    snprintf(buf, sizeof(buf)-2, "%s", getenv(STRINGIFY(ENVIRONMENT_HOME_DIR_NAME)));
+    gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER(filechooser), buf, NULL);
+  }
+
+  if (getcwd(buf, sizeof(buf)))
+  {
+    gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER(filechooser), buf, NULL);
+  }
+
+
+  if (enable_filters & XSANE_FILE_FILTER_ALL) /* filter: all files */
+  {
+   GtkFileFilter *filter;
+
+    filter = gtk_file_filter_new();
+    gtk_file_filter_add_pattern(filter, "*");
+
+    gtk_file_filter_set_name(filter, FILE_FILTER_ALL_FILES);
+    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filechooser), filter);
+
+    if (activate_filter == XSANE_FILE_FILTER_ALL)
+    {
+      gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filechooser), filter);
+    }
+  }
+
+  if (enable_filters & XSANE_FILE_FILTER_DRC) /* filter: device rc */
+  {
+   GtkFileFilter *filter;
+
+    filter = gtk_file_filter_new();
+    gtk_file_filter_add_pattern(filter, "*.[dD][rR][cC]");
+
+    gtk_file_filter_set_name(filter, FILE_FILTER_DRC);
+    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filechooser), filter);
+
+    if (activate_filter == XSANE_FILE_FILTER_DRC)
+    {
+      gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filechooser), filter);
+    }
+
+    /* add path to filechooser */
+    if (getenv(STRINGIFY(ENVIRONMENT_HOME_DIR_NAME)) != NULL)
+    {
+      gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER(filechooser), getenv(STRINGIFY(ENVIRONMENT_HOME_DIR_NAME)), NULL);
+    }
+  }
+
+  if (enable_filters & XSANE_FILE_FILTER_ICM) /* filter: color management profiles */
+  {
+   GtkFileFilter *filter;
+
+    filter = gtk_file_filter_new();
+    gtk_file_filter_add_pattern(filter, "*.[iI][cC][cCmM]");
+
+    gtk_file_filter_set_name(filter, FILE_FILTER_ICM);
+    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filechooser), filter);
+
+    if (activate_filter == XSANE_FILE_FILTER_ICM)
+    {
+      gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filechooser), filter);
+    }
+
+    /* add path to filechooser */
+    if (getenv(STRINGIFY(ENVIRONMENT_HOME_DIR_NAME)) != NULL)
+    {
+      snprintf(buf, sizeof(buf)-2, "%s%c.color%cicc", getenv(STRINGIFY(ENVIRONMENT_HOME_DIR_NAME)), SLASH, SLASH);
+      gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER(filechooser), buf, NULL);
+    }
+    gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER(filechooser), "/usr/share/color/icc", NULL);
+  }
+
+  if (enable_filters & XSANE_FILE_FILTER_IMAGES) /* filter: images */
+  {
+   GtkFileFilter *filter;
+
+    filter = gtk_file_filter_new();
+    gtk_file_filter_add_pattern(filter, "*.[jJ][pP][gG]");
+    gtk_file_filter_add_pattern(filter, "*.[jJ][pP][eE][gG]");
+    gtk_file_filter_add_pattern(filter, "*.[pP][nN][gG]");
+    gtk_file_filter_add_pattern(filter, "*.[tT][iI][fF]");
+    gtk_file_filter_add_pattern(filter, "*.[tT][iI][fF][fF]");
+    gtk_file_filter_add_pattern(filter, "*.[pP][sS]");
+    gtk_file_filter_add_pattern(filter, "*.[pP][dD][fF]");
+    gtk_file_filter_add_pattern(filter, "*.[pP][nN][mM]");
+    gtk_file_filter_add_pattern(filter, "*.[pP][bB][mM]");
+    gtk_file_filter_add_pattern(filter, "*.[pP][gG][mM]");
+    gtk_file_filter_add_pattern(filter, "*.[pP][pP][mM]");
+
+    gtk_file_filter_set_name(filter, FILE_FILTER_IMAGES);
+    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filechooser), filter);
+
+    if (activate_filter == XSANE_FILE_FILTER_IMAGES)
+    {
+      gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filechooser), filter);
+    }
+  }
+
+  if (enable_filters & XSANE_FILE_FILTER_BATCHLIST) /* filter: color management profiles */
+  {
+   GtkFileFilter *filter;
+
+    filter = gtk_file_filter_new();
+    gtk_file_filter_add_pattern(filter, "*.[xX][bV][lL]");
+
+    gtk_file_filter_set_name(filter, FILE_FILTER_XBL);
+    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filechooser), filter);
+
+    if (activate_filter == XSANE_FILE_FILTER_BATCHLIST)
+    {
+      gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filechooser), filter);
+    }
+  }
+
+  /* set default filename */
+  if (default_name) /* select file */
+  {
+   const char *basename = default_name;
+   char *path;
+   int pos;
+
+    DBG(DBG_info, "xsane_back_gtk_get_filename: default_name =%s\n", default_name);
+
+    path = strdup(default_name);
+    for (pos = strlen(path)-1; pos > 0; pos--)
+    {
+      if (path[pos] == '/')
+      {
+        path[pos]=0;
+
+	basename = path+pos+1;
+       break;
+      }
+    }
+
+    if (pos)
+    {
+      gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filechooser), path);
+    }
+
+    if ((action == XSANE_FILE_CHOOSER_ACTION_SAVE) || (action == XSANE_FILE_CHOOSER_ACTION_SELECT_SAVE))
+    {
+      gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filechooser), (char *) basename);
+    }
+    else
+    {
+      gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filechooser), (char *) default_name);
+    }
+  }
+
+
+  /* add filetype menu */
+
+  if (show_extra_widgets)
+  {
+   GtkWidget *vbox;
+   GtkWidget *hbox;
+   GtkWidget *label;
+
+    vbox = gtk_vbox_new(FALSE, 15);
+    gtk_widget_show(vbox);
+
+    if (show_extra_widgets & XSANE_GET_FILENAME_SHOW_FILETYPE)
+    {
+      DBG(DBG_info, "xsane_back_gtk_get_filename: showing filetype menu\n");
+  
+      if (filechooser_filetype)
+      {
+        free(filechooser_filetype);
+      }
+  
+      if ((filetype) && (*filetype))
+      {
+        filechooser_filetype = strdup(*filetype);
+      }
+      else
+      {
+        filechooser_filetype = NULL;
+      }
+  
+      hbox = gtk_hbox_new(FALSE, 2);
+      gtk_widget_show(hbox);
+      gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+  
+      label = gtk_label_new(TEXT_FILETYPE);
+      gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+      gtk_widget_show(label);
+  
+      xsane_filetype_option_menu = xsane_back_gtk_filetype_menu_new(filechooser_filetype, (GtkSignalFunc) xsane_back_gtk_filetype2_callback);
+      gtk_box_pack_start(GTK_BOX(hbox), xsane_filetype_option_menu, TRUE, TRUE, 2);
+      gtk_widget_show(xsane_filetype_option_menu);
+    }
+
+#ifdef HAVE_LIBLCMS 
+    if ((cms_function) && (show_extra_widgets & XSANE_GET_FILENAME_SHOW_CMS_FUNCTION))
+    {
+      hbox = gtk_hbox_new(FALSE, 2);
+      gtk_widget_show(hbox);
+      gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+  
+      label = gtk_label_new(TEXT_CMS_FUNCTION);
+      gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+      gtk_widget_show(label);
+  
+      xsane_cms_function_option_menu = xsane_back_gtk_cms_function_menu_new(*cms_function, NULL);
+      gtk_box_pack_start(GTK_BOX(hbox), xsane_cms_function_option_menu, TRUE, TRUE, 2);
+      gtk_widget_show(xsane_cms_function_option_menu);
+    }
+#endif
+
+    gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(filechooser), vbox);
+  }
+
+
+  gtk_widget_show(filechooser);
+
+  result = gtk_dialog_run(GTK_DIALOG(filechooser));
+
+  DBG(DBG_info, "xsane_back_gtk_get_filename: gtk_dialog_run() returned with result=%d\n", result);
+
+  if (result == accept_code)
+  {
+   char *chooser_filename;
+
+    if ((filetype) && (*filetype))
+    {
+      free(*filetype);
+      *filetype = NULL;
+    }
+
+    if (filechooser_filetype)
+    {
+      if (filetype)
+      {
+        *filetype = strdup(filechooser_filetype);
+      }
+    }
+
+#ifdef HAVE_LIBLCMS 
+    if ((cms_function) && (show_extra_widgets & XSANE_GET_FILENAME_SHOW_CMS_FUNCTION))
+    {
+      *cms_function = gtk_option_menu_get_history(GTK_OPTION_MENU(xsane_cms_function_option_menu));
+
+      DBG(DBG_info, "selected cms_function = %d\n", *cms_function);
+    }
+#endif
+
+    chooser_filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filechooser));
+    strncpy(filename, chooser_filename, max_len - 1);
+    g_free(chooser_filename);
+
+    filename[max_len - 1] = '\0';
+
+    ok = TRUE;
+  }
+
+  gtk_widget_destroy(filechooser);
+  filechooser = NULL;
+
+ return ok ? 0 : -1;
+}
+
+#else
+
+GtkWidget *fileselection;
+char *fileselection_filetype = NULL;
+
+/* ----------------------------------------------------------------------------------------------------------------- */
+
 static void xsane_back_gtk_get_filename_button_clicked(GtkWidget *w, gpointer data)
 {
  int *clicked = data;
@@ -1123,12 +1532,73 @@ static void xsane_back_gtk_get_filename_button_clicked(GtkWidget *w, gpointer da
 
 /* ----------------------------------------------------------------------------------------------------------------- */
 
-int xsane_back_gtk_get_filename(const char *label, const char *default_name, size_t max_len, char *filename, char **filetype,
-                                int show_fileopts, int shorten_path, int select_directory, int show_filetype_menu)
+static void xsane_back_gtk_filetype_callback(GtkWidget *widget, gpointer data)
+{
+ char *extension, *filename;
+ char buffer[PATH_MAX];
+ char *new_filetype = (char *) data;
+
+  DBG(DBG_proc, "xsane_filetype_callback\n");
+
+  filename = strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselection)));
+
+  if ((new_filetype) && (*new_filetype))
+  {
+    extension = strrchr(filename, '.');
+
+    if ((extension) && (extension != filename))
+    {
+      if ( (!strcasecmp(extension, ".pnm"))  || (!strcasecmp(extension, ".raw"))
+        || (!strcasecmp(extension, ".png"))  || (!strcasecmp(extension, ".ps"))
+        || (!strcasecmp(extension, ".pdf"))  || (!strcasecmp(extension, ".rgba"))
+        || (!strcasecmp(extension, ".tiff")) || (!strcasecmp(extension, ".tif"))
+        || (!strcasecmp(extension, ".text")) || (!strcasecmp(extension, ".txt"))
+        || (!strcasecmp(extension, ".jpg"))  || (!strcasecmp(extension, ".jpeg"))
+         ) /* remove filetype extension */
+      {
+        *extension = 0; /* remove extension */
+      }
+    }
+    snprintf(buffer, sizeof(buffer), "%s%s", filename, new_filetype);
+    free(filename);
+    filename = strdup(buffer);
+  }
+
+  if (fileselection_filetype)
+  {
+    free(fileselection_filetype);
+    fileselection_filetype = NULL;
+  }
+
+  if (data)
+  {
+    fileselection_filetype = strdup(new_filetype); 
+  }
+
+  gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection), filename);
+
+  free(filename);
+}
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
+int xsane_back_gtk_get_filename(const char *label, const char *default_name, size_t max_len, char *filename, char **filetype, int *cms_function,
+                                XsaneFileChooserAction action, int show_filetype_menu, int enable_filters, int activate_filter)
 {
  int cancel = 0, ok = 0, destroy = 0;
  GtkAccelGroup *accelerator_group;
  GtkWidget *xsane_filetype_option_menu;
+ int show_fileopts = 0;
+ int select_directory = 0;
+
+  if (action == XSANE_FILE_CHOOSER_ACTION_SELECT_FOLDER)
+  {
+    select_directory = TRUE;
+  }
+  else
+  {
+    show_fileopts = TRUE;
+  }
 
   DBG(DBG_proc, "xsane_back_gtk_get_filename\n");
 
@@ -1278,11 +1748,6 @@ int xsane_back_gtk_get_filename(const char *label, const char *default_name, siz
     cwd[cwd_len] = '\0';
 
     DBG(DBG_info, "xsane_back_gtk_get_filename: full path filename = %s\n", filename);
-    if (shorten_path && (strncmp(filename, cwd, cwd_len) == 0))
-    {
-      memcpy(filename, filename + cwd_len, len - cwd_len + 1);
-      DBG(DBG_info, "xsane_back_gtk_get_filename: short path filename = %s\n", filename);
-    }
   }
 
   if (!destroy)
@@ -1294,6 +1759,7 @@ int xsane_back_gtk_get_filename(const char *label, const char *default_name, siz
 
   return ok ? 0 : -1;
 }
+#endif
 
 /* ----------------------------------------------------------------------------------------------------------------- */
 
@@ -1435,8 +1901,37 @@ static void xsane_back_gtk_value_update(GtkAdjustment *adj_data, DialogElement *
 
   xsane_back_gtk_set_option(opt_num, &val, SANE_ACTION_SET_VALUE);
   xsane_control_option(xsane.dev, opt_num, SANE_ACTION_GET_VALUE, &new_val, 0);
+#if 1
+  val = new_val;
+  switch(opt->type)
+  {
+    case SANE_TYPE_INT:
+      if (new_val != val)
+      {
+        adj_data->value = val;
+        g_signal_emit_by_name(GTK_OBJECT(adj_data), "value_changed");
+      }
+     break;
 
-  if (new_val != val)
+    case SANE_TYPE_FIXED:
+      if (abs(new_val - val) > 1) /* tolarate 1/65536 error, instead of: if (new_val != val) */
+      {
+        d = SANE_UNFIX(val);
+        if (opt->unit == SANE_UNIT_MM)
+        {
+          d /= preferences.length_unit;
+        }
+        adj_data->value = d;
+        g_signal_emit_by_name(GTK_OBJECT(adj_data), "value_changed");
+      }
+     break;
+
+    default:
+     break;
+  }
+#endif
+#if 0
+  if (abs(new_val - val) > 1) /* tolarate 1/65536 error, instead of: if (new_val != val) */
   {
     val = new_val;
     switch(opt->type)
@@ -1459,6 +1954,7 @@ static void xsane_back_gtk_value_update(GtkAdjustment *adj_data, DialogElement *
     }
     g_signal_emit_by_name(GTK_OBJECT(adj_data), "value_changed");
   }
+#endif
 
  return;			/* value didn't change */
 }
@@ -1507,7 +2003,8 @@ void xsane_back_gtk_range_new(GtkWidget *parent, const char *name, gfloat val,
   label = gtk_label_new((char *) name);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
 
-  elem->data = gtk_adjustment_new(val, min, max, quant, quant*10, (max-min) * 1e-30);
+  elem->data = gtk_adjustment_new(val, min, max, quant, quant*10, 0);
+  //elem->data = gtk_adjustment_new(val, min, max, quant, quant*10, (max-min) * 1e-30);
   /* 1e-30 => hscrollbar has an unwanted side effect: the maximum is not the maximum */
   /* of the given range, it is reduced by the page_size, so it has to be very small */
 
@@ -1626,7 +2123,8 @@ void xsane_back_gtk_value_new(GtkWidget *parent, const char *name, gfloat val,
   label = gtk_label_new((char *) name);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
 
-  elem->data = gtk_adjustment_new(val, -1e29, 1e29, 1, 10, 1e-30);
+  elem->data = gtk_adjustment_new(val, -1e29, 1e29, 1, 10, 0);
+  //elem->data = gtk_adjustment_new(val, -1e29, 1e29, 1, 10, 1e-30);
 
   /* spinbutton */
 #ifndef HAVE_GTK2
@@ -2001,7 +2499,7 @@ void xsane_back_gtk_update_scan_window(void)
             GTK_ADJUSTMENT(elem->data)->value = new_val;
           }
 
-          if (old_val != new_val)
+          if (old_val != new_val) /* XXX dangerous comparison of doubles, we should allow tiny differences */
           {
 	      g_signal_emit_by_name(GTK_OBJECT(elem->data), "value_changed");
           }
@@ -2117,7 +2615,7 @@ void xsane_back_gtk_set_sensitivity(int sensitive)
       continue;
     }
 
-    if (!(opt->cap & SANE_CAP_ALWAYS_SETTABLE))
+  //  if (!(opt->cap & SANE_CAP_ALWAYS_SETTABLE))
     {
       gtk_widget_set_sensitive(xsane.element[i].widget, sensitive);
     }
