@@ -64,6 +64,7 @@ static gint xsane_email_dialog_delete();
 static void xsane_email_filetype_callback(GtkWidget *filetype_option_menu, char *filetype);
 static void xsane_email_receiver_changed_callback(GtkWidget *widget, gpointer data);
 static void xsane_email_subject_changed_callback(GtkWidget *widget, gpointer data);
+static void xsane_email_project_browse_filename_callback(GtkWidget *widget, gpointer data);
 static void xsane_email_project_changed_callback(GtkWidget *widget, gpointer data);
 static void xsane_email_html_mode_callback(GtkWidget *widget);
 static void xsane_email_project_display_status(void);
@@ -150,10 +151,8 @@ void xsane_email_dialog()
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 2);
   gtk_box_pack_start(GTK_BOX(email_scan_vbox), hbox, FALSE, FALSE, 1);
 
-  pixmap = gdk_pixmap_create_from_xpm_d(xsane.dialog->window, &mask, xsane.bg_trans, (gchar **) email_xpm);
-  pixmapwidget = gtk_image_new_from_pixmap(pixmap, mask);
-  gtk_box_pack_start(GTK_BOX(hbox), pixmapwidget, FALSE, FALSE, 2);
-  gdk_drawable_unref(pixmap);
+  button = xsane_button_new_with_pixmap(xsane.dialog->window, hbox, email_xpm, DESC_EMAIL_PROJECT_BROWSE,
+	                                                      (GtkSignalFunc) xsane_email_project_browse_filename_callback, NULL);
 
   text = gtk_entry_new();
   xsane_back_gtk_set_tooltip(xsane.tooltips, text, DESC_EMAIL_PROJECT);
@@ -165,7 +164,6 @@ void xsane_email_dialog()
   xsane.project_entry = text;
   xsane.project_entry_box = hbox;
 
-  gtk_widget_show(pixmapwidget);
   gtk_widget_show(text);
   gtk_widget_show(hbox);
 
@@ -185,6 +183,7 @@ void xsane_email_dialog()
   pixmapwidget = gtk_image_new_from_pixmap(pixmap, mask);
   gtk_box_pack_start(GTK_BOX(hbox), pixmapwidget, FALSE, FALSE, 2);
   gdk_drawable_unref(pixmap);
+  gdk_drawable_unref(mask);
 
   text = gtk_entry_new();
   gtk_entry_set_max_length(GTK_ENTRY(text), 128);
@@ -210,6 +209,7 @@ void xsane_email_dialog()
   pixmapwidget = gtk_image_new_from_pixmap(pixmap, mask);
   gtk_box_pack_start(GTK_BOX(hbox), pixmapwidget, FALSE, FALSE, 2);
   gdk_drawable_unref(pixmap);
+  gdk_drawable_unref(mask);
 
   text = gtk_entry_new();
   xsane_back_gtk_set_tooltip(xsane.tooltips, text, DESC_EMAIL_SUBJECT);
@@ -543,7 +543,7 @@ static void xsane_email_project_display_status()
       xsane.email_progress_val = val / 100.0;
       if (xsane.project_progress_bar)
       {
-        gtk_progress_bar_update(GTK_PROGRESS_BAR(xsane.project_progress_bar), xsane.email_progress_val);
+        xsane_progress_bar_set_fraction(GTK_PROGRESS_BAR(xsane.project_progress_bar), xsane.email_progress_val);
       }
 
       DBG(DBG_info, "reading from lockfile: email_status %s, email_progress_val %1.3f\n" , xsane.email_status, xsane.email_progress_val);
@@ -573,7 +573,7 @@ static void xsane_email_project_display_status()
     if (xsane.project_progress_bar)
     {
       gtk_progress_set_format_string(GTK_PROGRESS(xsane.project_progress_bar), _(xsane.email_status));
-      gtk_progress_bar_update(GTK_PROGRESS_BAR(xsane.project_progress_bar), xsane.email_progress_val);
+      xsane_progress_bar_set_fraction(GTK_PROGRESS_BAR(xsane.project_progress_bar), xsane.email_progress_val);
     }
   
     while (gtk_events_pending())
@@ -843,7 +843,7 @@ static void xsane_email_project_load()
   }
 
   gtk_progress_set_format_string(GTK_PROGRESS(xsane.project_progress_bar), _(xsane.email_status));
-  gtk_progress_bar_update(GTK_PROGRESS_BAR(xsane.project_progress_bar), xsane.email_progress_val);
+  xsane_progress_bar_set_fraction(GTK_PROGRESS_BAR(xsane.project_progress_bar), xsane.email_progress_val);
 
   xsane_email_project_display_status();
 
@@ -938,13 +938,13 @@ void xsane_email_project_save()
     snprintf(buf, 32, "%s@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@", xsane.email_status); /* fill 32 characters status line */
     fprintf(projectfile, "%s\n", buf); /* first line is status of email */
     gtk_progress_set_format_string(GTK_PROGRESS(xsane.project_progress_bar), _(xsane.email_status));
-    gtk_progress_bar_update(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
+    xsane_progress_bar_set_fraction(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
   }
   else
   {
     fprintf(projectfile, "                                \n"); /* no email status */
     gtk_progress_set_format_string(GTK_PROGRESS(xsane.project_progress_bar), "");
-    gtk_progress_bar_update(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
+    xsane_progress_bar_set_fraction(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
   }
 
   if (xsane.email_receiver)
@@ -1073,6 +1073,64 @@ static void xsane_email_subject_changed_callback(GtkWidget *widget, gpointer dat
 
 /* ---------------------------------------------------------------------------------------------------------------------- */
 
+void xsane_email_project_set_filename(gchar *filename)
+{
+  g_signal_handlers_block_by_func(GTK_OBJECT(xsane.project_entry), (GtkSignalFunc) xsane_email_project_changed_callback, NULL);
+  gtk_entry_set_text(GTK_ENTRY(xsane.project_entry), (char *) filename); /* update filename in entry */
+  gtk_entry_set_position(GTK_ENTRY(xsane.project_entry), strlen(filename)); /* set cursor to right position of filename */
+
+  g_signal_handlers_unblock_by_func(GTK_OBJECT(xsane.project_entry), (GtkSignalFunc) xsane_email_project_changed_callback, NULL);
+}
+
+/* ----------------------------------------------------------------------------------------------------------------- */
+
+static void xsane_email_project_browse_filename_callback(GtkWidget *widget, gpointer data)
+{
+ char filename[PATH_MAX];
+ char windowname[TEXTBUFSIZE];
+
+  DBG(DBG_proc, "xsane_email_project_browse_filename_callback\n");
+
+  xsane_set_sensitivity(FALSE);
+
+  if (preferences.email_project) /* make sure a correct filename is defined */
+  {
+    strncpy(filename, preferences.email_project, sizeof(filename));
+    filename[sizeof(filename) - 1] = '\0';
+  }
+  else /* no filename given, take standard filename */
+  {
+    strcpy(filename, OUT_FILENAME);
+  }
+
+  snprintf(windowname, sizeof(windowname), "%s %s %s", xsane.prog_name, WINDOW_EMAIL_PROJECT_BROWSE, xsane.device_text);
+
+  umask((mode_t) preferences.directory_umask); /* define new file permissions */
+  if (!xsane_back_gtk_get_filename(windowname, filename, sizeof(filename), filename, NULL, NULL, XSANE_FILE_CHOOSER_ACTION_SELECT_PROJECT, XSANE_GET_FILENAME_SHOW_NOTHING, 0, 0))
+  {
+
+    if (preferences.email_project)
+    {
+      free((void *) preferences.email_project);
+    }
+
+    preferences.email_project = strdup(filename);
+
+    xsane_set_sensitivity(TRUE);
+    xsane_email_project_set_filename(filename);
+
+    xsane_email_project_load();
+  }
+  else
+  {
+    xsane_set_sensitivity(TRUE);
+  }
+  umask(XSANE_DEFAULT_UMASK); /* define new file permissions */
+
+}
+
+/* ---------------------------------------------------------------------------------------------------------------------- */
+
 static void xsane_email_project_changed_callback(GtkWidget *widget, gpointer data)
 {
   DBG(DBG_proc, "xsane_email_project_changed_callback\n");
@@ -1248,16 +1306,15 @@ static void xsane_email_entry_rename_callback(GtkWidget *widget, gpointer list)
     xsane_set_window_icon(rename_dialog, 0);
 
     /* set the main vbox */
-    vbox = gtk_vbox_new(FALSE, 0);
-    gtk_container_set_border_width(GTK_CONTAINER(vbox), 0);
+    vbox = gtk_vbox_new(FALSE, 10);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
     gtk_container_add(GTK_CONTAINER(rename_dialog), vbox);
     gtk_widget_show(vbox);
 
     /* set the main hbox */
     hbox = gtk_hbox_new(FALSE, 0);
-    xsane_separator_new(vbox, 2);
-    gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
-    gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
+    gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(hbox), 0);
     gtk_widget_show(hbox); 
 
     gtk_window_set_position(GTK_WINDOW(rename_dialog), GTK_WIN_POS_CENTER);
@@ -1276,15 +1333,6 @@ static void xsane_email_entry_rename_callback(GtkWidget *widget, gpointer list)
     gtk_widget_show(text);
 
 
-#ifdef HAVE_GTK2
-    button = gtk_button_new_from_stock(GTK_STOCK_OK);
-#else
-    button = gtk_button_new_with_label(BUTTON_OK);
-#endif
-    g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_email_entry_rename_button_callback, (void *) 1);
-    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-    gtk_widget_show(button);
-
 #ifdef HAVE_GTK2
   button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
 #else
@@ -1294,6 +1342,15 @@ static void xsane_email_entry_rename_callback(GtkWidget *widget, gpointer list)
     gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
     gtk_widget_show(button);
 
+#ifdef HAVE_GTK2
+    button = gtk_button_new_from_stock(GTK_STOCK_OK);
+#else
+    button = gtk_button_new_with_label(BUTTON_OK);
+#endif
+    g_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc) xsane_email_entry_rename_button_callback, (void *) 1);
+    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+    gtk_widget_show(button);
+
 
     xsane_email_entry_rename = 0;
 
@@ -1428,7 +1485,7 @@ static void xsane_email_edit_callback(GtkWidget *widget, gpointer list)
     xsane_copy_file_by_name(outfilename, filename, xsane.multipage_progress_bar, &cancel_save);
 
     gtk_progress_set_format_string(GTK_PROGRESS(xsane.project_progress_bar), "");
-    gtk_progress_bar_update(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
+    xsane_progress_bar_set_fraction(GTK_PROGRESS_BAR(xsane.project_progress_bar), 0.0);
 
     xsane_viewer_new(outfilename, NULL, FALSE, filename, VIEWER_NO_NAME_MODIFICATION, IMAGE_SAVED);
   }
@@ -1666,7 +1723,7 @@ static void xsane_create_email(int fd)
     write_email_mime_ascii(fd, boundary);
 
     write(fd, email_text, strlen(email_text));
-    write(fd, "\r\n\r\n", 2);
+    write(fd, "\r\n\r\n", 4);
 
     for (i=0; i<attachments; i++)
     {
@@ -1856,7 +1913,7 @@ static void xsane_email_send()
     free(type);
     DBG(DBG_info, "converting %s to %s\n", source_filename, email_filename);
     output_format = xsane_identify_output_format(email_filename, NULL, NULL);
-    xsane_save_image_as(email_filename, source_filename, output_format, xsane.project_progress_bar, &cancel_save);
+    xsane_save_image_as(email_filename, source_filename, output_format, xsane.enable_color_management, preferences.cms_function, preferences.cms_intent, preferences.cms_bpc, xsane.project_progress_bar, &cancel_save);
     list = list->next;
     xsane.email_progress_size += xsane_get_filesize(email_filename);
   }
